#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['aiotmdb']

package_data = \
{'': ['*'], 'aiotmdb': ['resources/*']}

install_requires = \
['aiohttp']

extras_require = \
{'test': ['pytest >=2.7.3']}

setup(name='aiotmdb',
      version='0.3',
      description='Asyncio library for TMDb API',
      author='Fabrizio Arzeni',
      author_email='fabrizio.arz@gmail.com',
      url=None,
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
     )
